--include "lib\process.lua"
function cpyTable(tbl)
	local r={}
	
	if type(tbl)=="table" then
		for i, t in pairs(tbl) do
			r[i]=cpyTable(t)
		end
	else
		r=tbl
	end
	
	return r
end

function checkBounds(amount, data)--checks if bounds fall within any dimension array/set, amount has table index, data has minimum for the first #amount items, and then width, height, etc for next #amount items
	for i=1, #amount do
		if amount[i]<data[i] or amount[i]>=data[i]+data[i+#amount] then
			return false
		end
	end
	return true
end

function round(n)
	return math.floor(n+0.5)
end

local milSecs=1000--seconds to milliseconds
local minSecs=60--seconds to minutes

do menu=cpyTable(process)--name, item
	menu.items={}
	menu.backs={}
	menu.x=1
	menu.curBack=1
	menu.prevBackTime=millis()
	menu.backSecs=5
	local rectColor=0x00FF7F
	local textColor=0x000000
	
	function menu:reset()
		self.items={}
		self.back={}
		self.x=1
		self.curBack=1
		self.prevBackTime=millis()
		self.backSecs=5
	end
	
	function menu:setMenuItem(item)
		self.items=item.items
		self.backs=item.backs
		self.x=item.x
		self.curBack=item.curBack
		self.prevBackTime=item.prevBackTime
		self.backSecs=item.backSecs
	end
	
	function menu:drawBackground(gc)
		if self.backs[self.curBack]~=nil then
			gc:drawImage(self.backs[self.curBack], -1,0)
		end
	end
	
	function menu:arrowKey(key)
		if key=="up" then
			if self.x>1 then
				self.x=self.x-1
			else
				self.x=#self.items
			end
		elseif key=="down" then
			if self.x<#self.items then
				self.x=self.x+1
			else
				self.x=1
			end
		end
		disp:invalidate()
	end
	function menu:add(name, item)
		self.items[#self.items+1]={n=name, i=item, id=#self.items+1}
	end
	function menu:paint(gc)
		self:drawBackground(gc)
		gc:setFont("sansserif", "r", 14)
		for _, item in pairs(self.items) do
			gc:setColorRGB(textColor)
			centerText(gc, item.n, dispX/2, dispY/2-gc:getStringHeight(item.n)*(#self.items/2-item.id+1/2)*2/3)
			if item.id==self.x then
				gc:setColorRGB(rectColor)
				gc:drawRect(dispX/2-gc:getStringWidth(item.n)/2, dispY/2-gc:getStringHeight(item.n)*(#self.items/2-item.id+1)*2/3, gc:getStringWidth(item.n), gc:getStringHeight(item.n)*2/3)
			end
		end
		gc:setFont("sansserif", "r", 24)
		if self.title~=nil then
			if self.titleColor~=nil then
				gc:setColorRGB(self.titleColor)
			else
				gc:setColorRGB(textColor)
			end
			centerText(gc, self.title, dispX/2, gc:getStringHeight(self.title)/2, "top")
		end
	end
	
	function menu:setBackSecs(secs)
		self.backSecs=secs
	end
	
	function menu:timer()
		if millis()-self.prevBackTime>self.backSecs*1000 then
			if self.curBack<#self.backs then
				self.curBack=self.curBack+1
			else
				self.curBack=1
			end
			self.prevBackTime=millis()
		end
		disp:invalidate()
	end
	
	function menu:addBackground(img)
		self.backs[#self.backs+1]=img:copy(dispX+1, dispY)
	end
	
	function menu:activate()
		self:setDefaults()
	end
	
	function menu:Open()
		self.name="menu"
		self:setDefaults()
	end
	
	function menu:Close()
		self:deactivate()
	end
	
	function menu:enterKey()
		self:deactivate()
		if self.name==processes[#processes].name then
			PushProcess(self.items[self.x].i)
		end
		disp:invalidate()
	end
	function menu:mouseDown(x, y)
		self:enterKey()
	end
	
	function menu:setTitle(title, color)
		self.title=title
		self.titleColor=color
	end
end

do readme=cpyTable(process)
	readme.hlpInfo=""
	readme.dispHelp=D2Editor.newRichText()
	function readme:Open()
		--self:deactivate()
		--self.inputEnabled=true
		self.dispHelp:setVisible(false)
		self.dispHelp:setText(self.hlpInfo, 1)
		self.dispHelp:move(0, 0)
		self.dispHelp:setBorder(0)
		self.dispHelp:setReadOnly(true)
		self.dispHelp:resize(dispX, dispY)
		self.dispHelp:setVisible(true)
	end
	function readme:timer()
		self.dispHelp:setFocus(true)
	end
	function readme:Close()
		self.dispHelp:setFocus(false)
		self.dispHelp:setVisible(false)
	end
	function readme:escapeKey()
		PullProcess()
	end
	function readme:backspaceKey()
		PullProcess()
	end
	function readme:setText(text)
		self.hlpInfo=text
	end
end

do settings=cpyTable(process)--first param format: ant(current setting/amount/value/data), name(name of setting, like number mines), typ(type of setting, like "num" or "namelist"), numOn(if needed), data(if needed). second param: id(name of setting group, like game or menu or whatever)
	settings.list={}
	settings.index={}
	settings.x=1
	settings.curId=""
	
	function settings.getNum(key)
		print(key)
		if key=="left" then
			return -1
		elseif key=="right" then
			return 1
		end
		return 0
	end
	
	function getVals(lis, key)
		local numData={}
		
		for j, i in pairs(lis.data) do
			if type(i)=="function" then
				numData[j]=i(cpyTable(settings.list[settings.curId]), settings.getNum(key))
			else
				numData[j]=cpyTable(i)
			end
		end
		
		return numData
	end
	
	settings.functions={
		["num"]=function(lis, key)--data: inc(increment), min, max
			local numData=getVals(lis, key)
			local newAnt=lis.ant+numData["inc"]*settings.getNum(key)
			if numData["min"] and numData["min"]>round(newAnt) then
				newAnt=numData["max"]
			end
			if numData["max"] and numData["max"]<round(newAnt) then
				newAnt=numData["min"]
			end
			lis.ant=newAnt
			return lis
		end,
		["namelist"]=function(lis, key)--data["names"] is names
			print(lis.data["numOn"])
			if lis.data["numOn"]~=nil then
				lis.data["numOn"]=lis.data["numOn"]+settings.getNum(key)
			end
			if lis.data["numOn"]==nil or lis.data["numOn"]>#lis.data["names"] then
				lis.data["numOn"]=1
			elseif lis.data["numOn"]<1 then
				lis.data["numOn"]=#lis.data["names"]
			end
			print(lis.data["numOn"])
			lis.ant=lis.data["names"][lis.data["numOn"]]
			return cpyTable(lis)
		end
	}

	function settings:addFunc(f, typ)
		settings.functions[typ]=f
	end
	
	function settings:getFunc(typ)
		return settings.functions[typ]
	end
	
	function settings:getSetting(id)
		return cpyTable(self.list[id])
	end
	
	function settings:addSetting(sett, id)
		if not self.list[id] then
			self.list[id]={}
			self.index[id]={}
		end
		self.list[id][sett.n]=cpyTable(sett)
		self.index[id][#self.index[id]+1]=sett.n
		self.list[id][sett.n].id=#self.index[id]
		self.list[id][sett.n].enabled=true
	end
	
	function settings:addSettings(setts, id)
		self.curId=id
		for _, i in pairs(setts) do
			self:addSetting(i, id)
		end
	end
	
	function settings:setId(id)
		self.curId=id
	end
	
	function settings:arrowKey(key)
		print(self.x, #self.index[self.curId])
		if key=="down" then
			if self.x>1 then
				self.x=self.x-1
			else
				self.x=#self.index[self.curId]
			end
		elseif key=="up" then
			if self.x<#self.index[self.curId] then
				self.x=self.x+1
			else
				self.x=1
			end
		elseif key=="left" or key=="right" then
			local tmp=self.index[self.curId][self.x]
			if settings.functions[self.list[self.curId][tmp].typ] and self.list[self.curId][tmp].enabled then
				self.list[self.curId][tmp]=cpyTable(settings.functions[self.list[self.curId][tmp].typ](cpyTable(self.list[self.curId][tmp]), key))
			end
		end
		disp:invalidate()
		print(self.x, #self.index[self.curId])
	end
	
	function settings:paint(gc)
		for _, item in pairs(self.list[self.curId]) do
			gc:setColorRGB(0x000000)
			local j=item.ant
			if type(j)=="number" then
				j=round(j)
			end
			centerText(gc, item.name..": "..j, dispX/2, dispY/2+gc:getStringHeight(item.name..item.ant)*(#self.index[self.curId]/2-item.id)*2/3)
			if item.id==self.x then
				gc:drawRect(dispX/2-gc:getStringWidth(item.name..": "..j)/2, dispY/2+gc:getStringHeight(item.name..": "..j)*(#self.index[self.curId]/2-item.id-1/2)*2/3, gc:getStringWidth(item.name..": "..j), gc:getStringHeight(item.name..": "..j)*2/3)
			end
		end
	end
	
	function settings:escapeKey()
		PullProcess()
	end
end